#ifndef CTRIE_H_
#define CTRIE_H_

// Compact trie - internal nodes are collapsed together if they have no siblings.

struct ctrie;

struct ctrie * ctrie_new();
void ctrie_delete(struct ctrie * ctrie);

size_t ctrie_size(struct ctrie * ctrie);
size_t ctrie_count(struct ctrie * ctrie);

char * ctrie_insert(struct ctrie * ctrie, char * key, void * data);
void * ctrie_find(struct ctrie * ctrie, char * key);
void * ctrie_remove(struct ctrie * ctrie, char * key);

void ctrie_walk_keys(struct ctrie * ctrie, void (*op)(char * k, void * d) );

void ctrie_dump(struct ctrie * ctrie);
void ctrie_dump_raw(struct ctrie * ctrie);

struct list * ctrie_get_subkeys(struct ctrie * ctrie, char * prefix, size_t limit);
void          ctrie_free_subkey_list(struct list * list);


#endif
